<?
/**
 * File:  hrburl.php
 * url manipulation functions
 * Author:  Robocommerce, LLC (Zvi Boshernitzan and Brian Rosenthal)
 * Copyright 2004, 2005:  Robocommerce, LLC (http://www.robocommerce.com)
 * Distributed under Gnu General Public License (see includes/gpl_general.txt)
 * Version 1.0
 */

/**
 * @package rbcore
 */

/**
 * @package rbcore
 */
class rburl {

	function url() {$args = func_get_args(); return rburl::_url($args);}

	function _url($args) {
		$params = array();
		$param_mode = 'M'; // M (merge) - (subtract)  (TODO + (add))
		$ssl_p = 'AUTO'; // also SSL NOSSL
		$need_filename_p = true;
		// extract url-specific items

		foreach($args as $arg) {

			if (is_string($arg)) {

				switch ($arg) {

					case "":        break;

					case "SSL":
						case "NOSSL":
						case "AUTO":    $ssl_p      = $arg; break;

					case "M":
						case "-":       $param_mode = $arg; break;

					case "SELF":    $filename = basename(rb::php_self()); $need_filename_p = false; break;

					default:
					if ($need_filename_p) {
						$filename = $arg; $need_filename_p = false;
					} else {
						$arg = rburl::params_to_array($arg);
						if ($param_mode == '-') $arg = array_keys($arg);
					}
					break;
				}
			}
			if (is_array($arg)) {
				switch($param_mode) {
					case "M":       $params = array_merge($params,$arg); break;
					case "-":       $params = rba::rmkeys($params,$arg); $param_mode = "M"; break;
				}
			}
		} // end parsing args
		if ($ssl_p == "AUTO") $ssl_p = ($_SERVER['HTTPS'] == "on") ? "SSL" : "NOSSL";
		if (empty($filename)) $filename = $_SERVER['SCRIPT_URL'];

		$http_srv = (($ssl_p == 'SSL') and (ENABLE_SSL == true)) ? HTTPS_SERVER : HTTP_SERVER;

		$base_dir = '/';

		if (!preg_match("/^(http|\/)/i",$filename)) { // we need to add the prefixing stuff
			$filename = $http_srv . $base_dir . $filename;
		} else {
			$filename = str_replace(HTTP_SERVER,$http_srv,$filename);
		}
		$next_char = ((preg_match("/\?/", $filename) ? "&" : "?"));
		foreach($params as $key => $value) {
			if (!is_array($value)) {
				$filename .= $next_char . $key . "=" . urlencode($value);
				$next_char = "&";
			}
		}
		return $filename;
	}
	
	function to_params($arr,$exclude_vars='',$strip_empty_p=true) {
		if (!is_array($exclude_vars)) $exclude_vars = array();
		$exclude_vars = array_merge($exclude_vars,array("x","y","error"));
		$params = rba::rmkeys($arr,$exclude_vars,$strip_empty_p);
		return rburl::add_params("",$params,"");
	}

	function add_params($url,$params,$start_char="?") {
		// $params = rburl::params_to_array($params);
		$next_char = ((preg_match("/\?/", $url) ? "&" : $start_char));
		foreach($params as $key => $value) {
			if (is_array($value)) {
				foreach ($value as $subkey => $subval) {
					$url .= $next_char . $key . "[" . $subkey . "]=" . urlencode($subval);
					$next_char = "&";
				}
			}
			else {
				$url .= $next_char . $key . "=" . urlencode($value);
			}
			$next_char = "&";
		}
		return $url;
	}
	function params_to_array($args) {
		if (is_array($args)) return $args;
		if (is_string($args)) {
			$arg_array = array();
			parse_str($args,$arg_array);
			return $arg_array;
		}
		return array();
	}
	/** 
	 * does the currently executing script match this filename?
	 *  (only uses the last component of the filename)
	 * @return boolean
	 */
	function script_url_p($fn) {
		global $SCRIPT_URL;
		$script =  parse_url(rb::self());
		$url    =  parse_url($fn);
		return (array_pop(split("/", $script['path'])) == array_pop(split("/", $url['path'])));
	}
	/**
	 *
	 * @param string $name the new value to set (or exclude)
	 */
	function get_href($name = null, $new_value="", $new_base = null) {
		if (empty($name)) { return rburl::replace_params(array(),$new_base); }
		if (is_array($name)) return rburl::replace_params($name,$new_base);
		$arrvals = array($name => $new_value);
		return rburl::replace_params($arrvals, $new_base);
	}
	function replace_params($arrvals, $new_base = null) {
		$u = (empty($new_base) ? $_SERVER['SCRIPT_URL'] : $new_base);
		$sep = "?";
		// get everything from the request that is not in the array
		foreach ($_REQUEST as $key => $value) {
			if (!preg_match("/^(rb_admin_sid|dc|CVSSandbox|page_number|rbsid|browser_id|last_admin_username|CP)/", $key)) {
				if (!rba::striinarray($key, $arrvals)) {
					if (is_array($value)) {
						foreach ($value as $subkey => $subval) {
							$u .= $sep . $key . "[" . $subkey . "]=" . urlencode($subval);
							$sep = "&";
						}
					}
					else {
						$u .= $sep . $key . "=" . urlencode($value);
						$sep = "&";
					}
				}
			}
		}
		// include stuff from the array.
		foreach ($arrvals as $name => $new_value) {
			$u .= $sep . $name . "=" . urlencode($new_value);
			$sep = "&";
		}
		return $u;
	}
	function self() {
		$s  = ($_SERVER['HTTPS'] == "on") ? "https://" : "http://";
		$s .= $_SERVER['SERVER_NAME'];
		if (($_SERVER['SERVER_PORT'] <> 80) and ($_SERVER['SERVER_PORT'] <> 443))
			$s .= ":" . $_SERVER['SERVER_PORT'];
		return rburl::add_params($s . $_SERVER['REQUEST_URI'],$_POST);
	}

}

?>
